part of '../flutterx_live_data.dart';

/// Dispatcher for [LiveData] observers
abstract class Dispatcher {
  /// Schedule action on [SchedulerPhase.postFrameCallbacks] only if [SchedulerPhase.persistentCallbacks]
  static const Dispatcher postFrame = _PostFrameDispatcher();

  /// Schedule action after [SchedulerBinding.endOfFrame]
  static const Dispatcher endOfFrame = _EndOfFrameDispatcher();

  /// Schedule using [scheduleMicrotask] function
  static const Dispatcher microtask = _MicrotaskDispatcher();

  /// Dispatcher implementations also may be const
  const Dispatcher();

  /// Dispatch given [action] according to implementation context
  void post(VoidCallback action);
}

class _PostFrameDispatcher extends Dispatcher {
  const _PostFrameDispatcher();

  @override
  void post(VoidCallback action) => WidgetsBinding.instance.schedulerPhase == SchedulerPhase.persistentCallbacks
      ? WidgetsBinding.instance.addPostFrameCallback((_) => action())
      : action();
}

class _EndOfFrameDispatcher extends Dispatcher {
  const _EndOfFrameDispatcher();

  @override
  void post(VoidCallback action) => WidgetsBinding.instance.endOfFrame.then((_) => action());
}

class _MicrotaskDispatcher extends Dispatcher {
  const _MicrotaskDispatcher();

  @override
  void post(VoidCallback action) => scheduleMicrotask(action);
}
